@@ -7,6 +7,7 @@ use self::handle::BodyHandle;
 use std::convert::TryInto;
 use std::fmt::Debug;
 use std::io::{BufRead, BufReader, BufWriter, Read, Write};
+use std::mem::{self, ManuallyDrop};
 
 pub use streaming::StreamingBody;
 
@@ -19,7 +20,7 @@ pub use streaming::StreamingBody;
 /// control over aspects of the buffering using the [`BufRead`] methods and [`Write::flush()`].
 pub struct Body {
     reader: BufReader<BodyHandle>,
-    writer: BufWriter<BodyHandle>,
+    writer: BufWriter<BodyHandleWrapper>,
 }
 
 impl Debug for Body {
@@ -54,7 +55,8 @@ impl Body {
             // We have to cheat a little here to get mutable access to the handle while the reader
             // buffer is borrowed. Since we're not going to read or write through the `self`
             // interface while `body_handle` is live, no other aliases of the handle will be used.
-            let mut body_handle = unsafe { BodyHandle::from_u32(self.reader.get_ref().as_u32()) };
+            let mut body_handle =
+                ManuallyDrop::new(unsafe { BodyHandle::from_u32(self.reader.get_ref().as_u32()) });
             let nwritten = body_handle.write_front(read_buf);
             drop(read_buf);
             // Let the `BufReader` know that we've consumed these bytes from its internal buffer so
@@ -65,38 +67,25 @@ impl Body {
 
     /// Read the entirety of the body into a byte vector.
     ///
-    /// # Memory usage
-    ///
-    /// This method will cause the entire body to be buffering in WebAssembly memory. You should
-    /// take care not to exceed the WebAssembly memory limits, and consider using methods like
-    /// [`BufRead::lines()`] or [`Body::read_chunks()`] to control how much of the body you process
-    /// at once.
+    #[doc = include_str!("../../docs/snippets/buffers-body.md")]
     pub fn into_bytes(self) -> Vec<u8> {
         self.into_handle().into_bytes()
     }
 
     /// Read the entirety of the body into a `String`, interpreting the bytes as UTF-8.
     ///
-    /// # Memory usage
-    ///
-    /// This method will cause the entire body to be buffering in WebAssembly memory. You should
-    /// take care not to exceed the WebAssembly memory limits, and consider using methods like
-    /// [`BufRead::lines()`] or [`Body::read_chunks()`] to control how much of the body you process
-    /// at once.
+    #[doc = include_str!("../../docs/snippets/buffers-body.md")]
     ///
     /// # Panics
     ///
-    /// If the body does not contain a valid UTF-8 string, this function will panic. To explicitly
-    /// handle the possibility of invalid UTF-8 data, use [`into_bytes()`][`Self::into_bytes()`] and
-    /// then convert the bytes explicitly with a function like [`String::from_utf8`].
+    #[doc = include_str!("../../docs/snippets/panics-body-utf8.md")]
     pub fn into_string(self) -> String {
         self.into_handle().into_string()
     }
 
     /// Append another body onto the end of this body.
     ///
-    /// This operation is performed in amortized constant time, and so should always be preferred to
-    /// reading an entire body and then writing the same contents to another body.
+    #[doc = include_str!("../../docs/snippets/body-append-constant-time.md")]
     pub fn append(&mut self, other: Body) {
         // flush the write buffer of the destination body, so that we can use the append method on
         // the underlying handles. Unwrap, as `BodyHandle::flush` won't return actionable errors.
@@ -194,7 +183,7 @@ impl Body {
     /// ```no_run
     /// const MAGIC: &[u8] = b"\0asm";
     /// # let mut body = fastly::Body::from(MAGIC);
-    /// let prefix = body.get_prefix_mut(2);
+    /// let prefix = body.get_prefix_mut(MAGIC.len());
     /// if prefix.as_slice() == MAGIC {
     ///     println!("might be Wasm!");
     /// }
@@ -400,7 +389,7 @@ impl From<BodyHandle> for Body {
         let handle2 = unsafe { BodyHandle::from_u32(handle.as_u32()) };
         Self {
             reader: BufReader::new(handle),
-            writer: BufWriter::new(handle2),
+            writer: BufWriter::new(BodyHandleWrapper::new(handle2)),
         }
     }
 }
@@ -541,3 +530,52 @@ impl<'a> Drop for PrefixString<'a> {
         }
     }
 }
+
+/// An internal wrapper used in `Body` to prevent closing the handle twice by
+/// wrapping a `BodyHandle` in this type with a special Drop impl to prevent the
+/// `BodyHandle` having it's destructor called. This type should not be used outside
+/// of this module. All the function calls used by `Body` on the writer handle
+/// are reimplemented for the wrapper and it just passes the function call to
+/// the inner handle
+#[repr(transparent)]
+struct BodyHandleWrapper {
+    handle: BodyHandle,
+}
+
+impl Drop for BodyHandleWrapper {
+    fn drop(&mut self) {
+        // Weirdly having the handle field be ManuallyDrop<BodyHandle> was
+        // causing issues so we replace it with an invalid handle which will not
+        // drop the resource in BodyHandle's Drop impl and then we wrap the
+        // actual handle we get back with ManuallyDrop which will just be a u32
+        // on the stack that will get cleared out so we don't leak any memory
+        let handle = mem::replace(&mut self.handle, unsafe {
+            BodyHandle::from_u32(fastly_shared::INVALID_BODY_HANDLE)
+        });
+        mem::forget(handle);
+    }
+}
+
+impl BodyHandleWrapper {
+    fn new(handle: BodyHandle) -> Self {
+        Self { handle }
+    }
+
+    fn write_front(&mut self, buf: &[u8]) -> usize {
+        self.handle.write_front(buf)
+    }
+}
+
+impl Write for BodyHandleWrapper {
+    fn write(&mut self, buf: &[u8]) -> std::io::Result<usize> {
+        self.handle.write(buf)
+    }
+
+    fn write_vectored(&mut self, bufs: &[std::io::IoSlice<'_>]) -> std::io::Result<usize> {
+        self.handle.write_vectored(bufs)
+    }
+
+    fn flush(&mut self) -> std::io::Result<()> {
+        self.handle.flush()
+    }
+}
