@@ -1,9 +1,15 @@
 //! HTTP bodies.
 
-use crate::abi;
+use crate::{
+    abi::{self, FastlyStatus},
+    error::{HandleError, HandleKind},
+};
 use fastly_shared::BodyWriteEnd;
-use std::io::{BufReader, Read, Write};
 use std::sync::atomic::{AtomicBool, Ordering};
+use std::{
+    io::{BufReader, Read, Write},
+    mem::ManuallyDrop,
+};
 
 /// A low-level interface to HTTP bodies.
 ///
@@ -32,20 +38,20 @@ impl BodyHandle {
     };
 
     /// Returns `true` if the body handle is valid.
-    pub fn is_valid(&self) -> bool {
+    pub const fn is_valid(&self) -> bool {
         !self.is_invalid()
     }
 
     /// Returns `true` if the body handle is invalid.
-    pub fn is_invalid(&self) -> bool {
-        self.handle == Self::INVALID.handle
+    pub const fn is_invalid(&self) -> bool {
+        self.handle == fastly_shared::INVALID_BODY_HANDLE
     }
 
     /// Make a handle from its underlying representation.
     ///
     /// This should only be used when calling the raw ABI directly, and care should be taken not to
     /// reuse or alias handle values.
-    pub(crate) fn from_u32(handle: u32) -> Self {
+    pub unsafe fn from_u32(handle: u32) -> Self {
         Self { handle }
     }
 
@@ -53,7 +59,7 @@ impl BodyHandle {
     ///
     /// This should only be used when calling the raw ABI directly, and care should be taken not to
     /// reuse or alias handle values.
-    pub(crate) unsafe fn as_u32(&self) -> u32 {
+    pub unsafe fn as_u32(&self) -> u32 {
         self.handle
     }
 
@@ -61,10 +67,18 @@ impl BodyHandle {
     ///
     /// This should only be used when calling the raw ABI directly, and care should be taken not to
     /// reuse or alias handle values.
-    pub(crate) fn as_u32_mut(&mut self) -> &mut u32 {
+    pub unsafe fn as_u32_mut(&mut self) -> &mut u32 {
         &mut self.handle
     }
 
+    /// Turn a handle into its representation without closing the underlying resource.
+    ///
+    /// This should only be used when calling the raw ABI directly, and care should be taken not to
+    /// reuse or alias handle values.
+    pub fn into_u32(self) -> u32 {
+        unsafe { ManuallyDrop::new(self).as_u32() }
+    }
+
     /// Set `GOT_CLIENT_BODY` flag to show we've taken the client body.
     ///
     /// This will panic if the flag has already been set by someone else.
@@ -104,23 +118,18 @@ impl BodyHandle {
 
     /// Append another body onto the end of this body.
     ///
-    /// This operation is performed in amortized constant time, and so should always be preferred to
-    /// reading an entire body and then writing the same contents to another body.
+    #[doc = include_str!("../../../docs/snippets/body-append-constant-time.md")]
     ///
     /// The other body will no longer be valid after this call.
     pub fn append(&mut self, other: BodyHandle) {
-        unsafe { abi::fastly_http_body::append(self.as_u32(), other.as_u32()) }
+        unsafe { abi::fastly_http_body::append(self.as_u32(), other.into_u32()) }
             .result()
             .expect("fastly_http_body::append failed")
     }
 
     /// Read the entirety of the body into a byte vector.
     ///
-    /// # Memory usage
-    ///
-    /// This method will cause the entire body to be buffered in WebAssembly memory. You should take
-    /// care not to exceed the WebAssembly memory limits, and consider using [`Read`] methods to
-    /// control how much of the body you process at once.
+    #[doc = include_str!("../../../docs/snippets/buffers-body-handle.md")]
     pub fn into_bytes(self) -> Vec<u8> {
         let mut body = vec![];
         let mut bufread = BufReader::new(self);
@@ -132,17 +141,11 @@ impl BodyHandle {
 
     /// Read the entirety of the body into a `String`, interpreting the bytes as UTF-8.
     ///
-    /// # Memory usage
-    ///
-    /// This method will cause the entire body to be buffered in WebAssembly memory. You should take
-    /// care not to exceed the WebAssembly memory limits, and consider using [`Read`] methods to
-    /// control how much of the body you process at once.
+    #[doc = include_str!("../../../docs/snippets/buffers-body-handle.md")]
     ///
     /// # Panics
     ///
-    /// If the body does not contain a valid UTF-8 string, this function will panic. To explicitly
-    /// handle the possibility of invalid UTF-8 data, use [`into_bytes()`][`Self::into_bytes()`] and
-    /// then convert the bytes explicitly with a function like [`String::from_utf8`].
+    #[doc = include_str!("../../../docs/snippets/panics-body-utf8.md")]
     pub fn into_string(self) -> String {
         let mut body = String::new();
         let mut bufread = BufReader::new(self);
@@ -209,6 +212,32 @@ impl BodyHandle {
         );
         nwritten
     }
+
+    /// Close the BodyHandle by removing it from the host Session. This will close
+    /// out streaming and non streaming bodies. Care must be taken to only
+    /// close out the handle if the body is not being written to or read from
+    /// anymore. If the handle has already been closed an error will be
+    /// returned.
+    ///
+    /// # Examples
+    ///
+    /// ```no_run
+    /// # use fastly::handle::BodyHandle;
+    /// # fn main() -> Result<(), Box<dyn std::error::Error>> {
+    /// let mut body = BodyHandle::new();
+    /// body.write_str("You're already closed.");
+    /// // The handle is not being used in a request and doesn't refer to any
+    /// // response body so we can close this out
+    /// body.close()?;
+    /// # Ok(())
+    /// # }
+    /// ```
+    pub fn close(self) -> Result<(), HandleError> {
+        match unsafe { abi::fastly_http_body::close(self.into_u32()) } {
+            FastlyStatus::OK => Ok(()),
+            _ => Err(HandleError::ClosedHandle(HandleKind::Body)),
+        }
+    }
 }
 
 impl Read for BodyHandle {
@@ -219,13 +248,15 @@ impl Read for BodyHandle {
         let status = unsafe {
             abi::fastly_http_body::read(self.as_u32(), buf.as_mut_ptr(), buf.len(), &mut nread)
         };
-        if status.is_err() {
-            Err(Error::new(
+        match status {
+            FastlyStatus::OK => Ok(nread),
+            FastlyStatus::HTTPINCOMPLETE => {
+                Err(Error::new(ErrorKind::UnexpectedEof, "incomplete HTTP body"))
+            }
+            _ => Err(Error::new(
                 ErrorKind::Other,
                 "fastly_http_body::read failed",
-            ))
-        } else {
-            Ok(nread)
+            )),
         }
     }
 }
@@ -240,6 +271,16 @@ impl Write for BodyHandle {
     }
 }
 
+impl Drop for BodyHandle {
+    fn drop(&mut self) {
+        if self.is_valid() {
+            unsafe { abi::fastly_http_body::close(self.as_u32()) }
+                .result()
+                .expect("fastly_http_body::close failed");
+        }
+    }
+}
+
 impl From<&str> for BodyHandle {
     fn from(s: &str) -> Self {
         let mut handle = Self::new();
