@@ -1,6 +1,6 @@
 //! HTTP requests.
 
-use self::handle::RequestHandle;
+use self::handle::{ContentEncodings, RequestHandle};
 use super::body::{self, Body, StreamingBody};
 use super::response::{handles_to_response, FastlyResponseMetadata, Response};
 use crate::backend::Backend;
@@ -97,6 +97,7 @@ pub struct Request {
     body: Option<Body>,
     cache_override: CacheOverride,
     is_from_client: bool,
+    auto_decompress_response: ContentEncodings,
     // Overridden via experimenal::RequestCacheKey
     pub(crate) cache_key: Option<CacheKeyGen>,
 }
@@ -137,8 +138,13 @@ impl Request {
     /// [`try_from_client()`][`Self::try_from_client()`] if you want to explicitly handle these
     /// errors, for example by returning a customized error page.
     pub fn from_client() -> Request {
-        Request::try_from_client()
-            .unwrap_or_else(|_| panic_with_status!(crate::http::StatusCode::BAD_REQUEST))
+        Request::try_from_client().unwrap_or_else(|e| {
+            panic_with_status!(
+                crate::http::StatusCode::BAD_REQUEST,
+                "fastly::limits::RequestLimits exceeded: {}",
+                e
+            )
+        })
     }
 
     /// Get the client request being handled by this execution of the Compute@Edge program, or an
@@ -175,6 +181,7 @@ impl Request {
             body: None,
             cache_override: CacheOverride::default(),
             is_from_client: false,
+            auto_decompress_response: ContentEncodings::empty(),
             cache_key: None,
         }
     }
@@ -209,6 +216,7 @@ impl Request {
             body: None,
             cache_override: self.cache_override.clone(),
             is_from_client: self.is_from_client,
+            auto_decompress_response: self.auto_decompress_response,
             cache_key: self.cache_key.clone(),
         }
     }
@@ -219,9 +227,7 @@ impl Request {
     /// This method requires mutable access to this request because reading from and writing to the
     /// body can involve an HTTP connection.
     ///
-    /// This operation is potentially expensive if the body is large. Take care when using this
-    /// method on bodies with unknown sizes. Consider using methods like [`BufRead::lines()`] or
-    /// [`Body::read_chunks()`] to incrementally process a body while limiting the maximum size.
+    #[doc = include_str!("../../docs/snippets/clones-body.md")]
     ///
     /// # Examples
     ///
@@ -251,90 +257,63 @@ impl Request {
 
     /// Create a new `GET` [`Request`] with the given URL, no headers, and an empty body.
     ///
-    /// # Argument type conversion
-    ///
-    /// The URL argument can be any type that implements [`ToUrl`]; see that trait for details on
-    /// which types can be used and when panics may arise during conversion.
+    #[doc = include_str!("../../docs/snippets/url-argument.md")]
     pub fn get(url: impl ToUrl) -> Self {
         Self::new(Method::GET, url)
     }
 
     /// Create a new `HEAD` [`Request`] with the given URL, no headers, and an empty body.
     ///
-    /// # Argument type conversion
-    ///
-    /// The URL argument can be any type that implements [`ToUrl`]; see that trait for details on
-    /// which types can be used and when panics may arise during conversion.
+    #[doc = include_str!("../../docs/snippets/url-argument.md")]
     pub fn head(url: impl ToUrl) -> Self {
         Self::new(Method::HEAD, url)
     }
 
     /// Create a new `POST` [`Request`] with the given URL, no headers, and an empty body.
     ///
-    /// # Argument type conversion
-    ///
-    /// The URL argument can be any type that implements [`ToUrl`]; see that trait for details on
-    /// which types can be used and when panics may arise during conversion.
+    #[doc = include_str!("../../docs/snippets/url-argument.md")]
     pub fn post(url: impl ToUrl) -> Self {
         Self::new(Method::POST, url)
     }
 
     /// Create a new `PUT` [`Request`] with the given URL, no headers, and an empty body.
     ///
-    /// # Argument type conversion
-    ///
-    /// The URL argument can be any type that implements [`ToUrl`]; see that trait for details on
-    /// which types can be used and when panics may arise during conversion.
+    #[doc = include_str!("../../docs/snippets/url-argument.md")]
     pub fn put(url: impl ToUrl) -> Self {
         Self::new(Method::PUT, url)
     }
 
     /// Create a new `DELETE` [`Request`] with the given URL, no headers, and an empty body.
     ///
-    /// # Argument type conversion
-    ///
-    /// The URL argument can be any type that implements [`ToUrl`]; see that trait for details on
-    /// which types can be used and when panics may arise during conversion.
+    #[doc = include_str!("../../docs/snippets/url-argument.md")]
     pub fn delete(url: impl ToUrl) -> Self {
         Self::new(Method::DELETE, url)
     }
 
     /// Create a new `CONNECT` [`Request`] with the given URL, no headers, and an empty body.
     ///
-    /// # Argument type conversion
-    ///
-    /// The URL argument can be any type that implements [`ToUrl`]; see that trait for details on
-    /// which types can be used and when panics may arise during conversion.
+    #[doc = include_str!("../../docs/snippets/url-argument.md")]
     pub fn connect(url: impl ToUrl) -> Self {
         Self::new(Method::CONNECT, url)
     }
 
     /// Create a new `OPTIONS` [`Request`] with the given URL, no headers, and an empty body.
     ///
-    /// # Argument type conversion
-    ///
-    /// The URL argument can be any type that implements [`ToUrl`]; see that trait for details on
-    /// which types can be used and when panics may arise during conversion.
+    #[doc = include_str!("../../docs/snippets/url-argument.md")]
     pub fn options(url: impl ToUrl) -> Self {
         Self::new(Method::OPTIONS, url)
     }
 
     /// Create a new `TRACE` [`Request`] with the given URL, no headers, and an empty body.
     ///
-    /// # Argument type conversion
-    ///
-    /// The URL argument can be any type that implements [`ToUrl`]; see that trait for details on
-    /// which types can be used and when panics may arise during conversion.
+    #[doc = include_str!("../../docs/snippets/url-argument.md")]
     pub fn trace(url: impl ToUrl) -> Self {
         Self::new(Method::TRACE, url)
     }
 
     /// Create a new `PATCH` [`Request`] with the given URL, no headers, and an empty body.
     ///
-    /// # Argument type conversion
-    ///
-    /// The URL argument can be any type that implements [`ToUrl`]; see that trait for details on
-    /// which types can be used and when panics may arise during conversion.
+    #[doc = include_str!("../../docs/snippets/url-argument.md")]
     pub fn patch(url: impl ToUrl) -> Self {
         Self::new(Method::PATCH, url)
     }
@@ -342,15 +321,11 @@ impl Request {
     /// Send the request to the given backend server, and return once the response headers have been
     /// received, or an error occurs.
     ///
-    /// # Argument type conversion
-    ///
-    /// The backend argument can be any type that implements [`ToBackend`]; see that trait for
-    /// details on which types can be used and when panics may arise during conversion.
+    #[doc = include_str!("../../docs/snippets/backend-argument.md")]
     ///
     /// # Panics
     ///
-    /// Panics if the backend response exceeds any of the limits configured by
-    /// [`ResponseLimits`][`crate::limits::ResponseLimits`].
+    #[doc = include_str!("../../docs/snippets/panics-responselimits.md")]
     ///
     /// # Examples
     ///
@@ -400,10 +375,7 @@ impl Request {
     /// request may take longer than the Compute@Edge program is able to run, as the request will
     /// continue sending even after the program that intitiated it exits.
     ///
-    /// # Argument type conversion
-    ///
-    /// The backend argument can be any type that implements [`ToBackend`]; see that trait for
-    /// details on which types can be used and when panics may arise during conversion.
+    #[doc = include_str!("../../docs/snippets/backend-argument.md")]
     ///
     /// # Examples
     ///
@@ -551,8 +523,7 @@ impl Request {
 
     /// Get a mutable reference to the body of this request.
     ///
-    /// An empty body is returned if no body has been set, or if it has previously been returned by
-    /// a method like [`take_body()`][`Self::take_body()`].
+    #[doc = include_str!("../../docs/snippets/creates-empty-body.md")]
     ///
     /// # Examples
     ///
@@ -622,13 +593,8 @@ impl Request {
     }
 
     /// Set the given value as the request's body.
-    ///
-    /// Any previous body that may have been set on the response is discarded. To add to an existing
-    /// body, use [`get_body_mut()`][`Self::get_body_mut()`] and write to the returned [`Body`].
-    ///
-    /// # Argument type conversion
-    ///
-    /// See [`Body`] for details on which types can be used as a body.
+    #[doc = include_str!("../../docs/snippets/body-argument.md")]
+    #[doc = include_str!("../../docs/snippets/discards-body.md")]
     pub fn set_body(&mut self, body: impl Into<Body>) {
         self.body = Some(body.into());
     }
@@ -637,8 +603,7 @@ impl Request {
     ///
     /// After calling this method, this request will no longer have a body.
     ///
-    /// An empty body is returned if no body has been set, or if it has previously been returned by
-    /// a method like [`take_body()`][`Self::take_body()`].
+    #[doc = include_str!("../../docs/snippets/creates-empty-body.md")]
     pub fn take_body(&mut self) -> Body {
         self.body.take().unwrap_or_else(|| Body::new())
     }
@@ -655,8 +620,7 @@ impl Request {
     ///
     /// If this request does not have a body, the appended body is set as the request's body.
     ///
-    /// This operation is performed in amortized constant time, and so should always be preferred to
-    /// reading an entire body and then writing the same contents to another body.
+    #[doc = include_str!("../../docs/snippets/body-append-constant-time.md")]
     ///
     /// This method should be used when combining bodies that have not necessarily been read yet,
     /// such as the body of the client. To append contents that are already in memory as strings or
@@ -681,13 +645,7 @@ impl Request {
 
     /// Consume the request and return its body as a byte vector.
     ///
-    /// # Memory usage
-    ///
-    /// This method will cause the entire body to be buffering in WebAssembly memory. You should
-    /// take care not to exceed the WebAssembly memory limits, and consider using methods like
-    /// [`read_body_lines()`][`Self::read_body_lines()`] or
-    /// [`read_body_chunks()`][`Self::read_body_chunks()`] to control how much of the body you
-    /// process at once.
+    #[doc = include_str!("../../docs/snippets/buffers-body-reqresp.md")]
     ///
     /// # Examples
     ///
@@ -702,20 +660,11 @@ impl Request {
 
     /// Consume the request and return its body as a string.
     ///
-    /// # Memory usage
-    ///
-    /// This method will cause the entire body to be buffering in WebAssembly memory. You should
-    /// take care not to exceed the WebAssembly memory limits, and consider using methods like
-    /// [`read_body_lines()`][`Self::read_body_lines()`] or
-    /// [`read_body_chunks()`][`Self::read_body_chunks()`] to control how much of the body you
-    /// process at once.
+    #[doc = include_str!("../../docs/snippets/buffers-body-reqresp.md")]
     ///
     /// # Panics
     ///
-    /// If the body does not contain a valid UTF-8 string, this function will panic. To explicitly
-    /// handle the possibility of invalid UTF-8 data, use
-    /// [`into_body_bytes()`][`Self::into_body_bytes()`] and then convert the bytes explicitly with
-    /// a function like [`String::from_utf8`].
+    #[doc = include_str!("../../docs/snippets/panics-reqresp-intobody-utf8.md")]
     ///
     /// # Examples
     ///
@@ -731,8 +680,7 @@ impl Request {
 
     /// Consume the request and return its body.
     ///
-    /// An empty body is returned if no body has been set, or if it has previously been returned by
-    /// a method like [`take_body()`][`Self::take_body()`].
+    #[doc = include_str!("../../docs/snippets/creates-empty-body.md")]
     pub fn into_body(self) -> Body {
         self.body.unwrap_or_else(|| Body::new())
     }
@@ -762,12 +710,8 @@ impl Request {
 
     /// Set the given string as the request's body with content type `text/plain; charset=UTF-8`.
     ///
-    /// Any previous body that may have been set on the response is discarded. To add to an existing
-    /// body, use [`get_body_mut()`][`Self::get_body_mut()`] and write to the returned [`Body`].
-    ///
-    /// # Content type
-    ///
-    /// This method sets the content type to `text/plain; charset=utf-8`.
+    #[doc = include_str!("../../docs/snippets/discards-body.md")]
+    #[doc = include_str!("../../docs/snippets/sets-text-plain.md")]
     ///
     /// # Examples
     ///
@@ -791,12 +735,8 @@ impl Request {
 
     /// Set the given string as the request's body with content type `text/html; charset=UTF-8`.
     ///
-    /// Any previous body that may have been set on the response is discarded. To add to an existing
-    /// body, use [`get_body_mut()`][`Self::get_body_mut()`] and write to the returned [`Body`].
-    ///
-    /// # Content type
-    ///
-    /// This method sets the content type to `text/html; charset=utf-8`.
+    #[doc = include_str!("../../docs/snippets/discards-body.md")]
+    #[doc = include_str!("../../docs/snippets/sets-text-html.md")]
     ///
     /// # Examples
     ///
@@ -816,19 +756,11 @@ impl Request {
     ///
     /// After calling this method, this request will no longer have a body.
     ///
-    /// # Memory usage
-    ///
-    /// This method will cause the entire body to be buffering in WebAssembly memory. You should
-    /// take care not to exceed the WebAssembly memory limits, and consider using methods like
-    /// [`read_body_lines()`][`Self::read_body_lines()`] or
-    /// [`read_body_chunks()`][`Self::read_body_chunks()`] to control how much of the body you
-    /// process at once.
+    #[doc = include_str!("../../docs/snippets/buffers-body-reqresp.md")]
     ///
     /// # Panics
     ///
-    /// If the body does not contain a valid UTF-8 string, this function will panic. To handle the
-    /// possibility of invalid UTF-8 data, use [`take_body_bytes()`][`Self::take_body_bytes()`] and
-    /// then convert the bytes with a function like [`String::from_utf8`].
+    #[doc = include_str!("../../docs/snippets/panics-reqresp-takebody-utf8.md")]
     ///
     /// # Examples
     ///
@@ -867,31 +799,45 @@ impl Request {
         self.get_body_mut().lines()
     }
 
-    /// Builder-style equivalent of [`set_body_bytes()`][`Self::set_body_bytes()`].
-    pub fn with_body_bytes(mut self, body: &[u8]) -> Self {
-        self.set_body_bytes(body);
+    /// Deprecated alias of [`with_body_octet_stream()`][`Self::with_body_octet_stream()`]
+    #[deprecated(
+        since = "0.8.1",
+        note = "renamed to `Request::with_body_octet_stream()`"
+    )]
+    pub fn with_body_bytes(self, body: &[u8]) -> Self {
+        self.with_body_octet_stream(body)
+    }
+
+    /// Deprecated alias of [`set_body_octet_stream()`][`Self::set_body_octet_stream()`]
+    #[deprecated(
+        since = "0.8.1",
+        note = "renamed to `Request::set_body_octet_stream()`"
+    )]
+    pub fn set_body_bytes(&mut self, body: &[u8]) {
+        self.set_body_octet_stream(body);
+    }
+
+    /// Builder-style equivalent of [`set_body_octet_stream()`][`Self::set_body_octet_stream()`].
+    pub fn with_body_octet_stream(mut self, body: &[u8]) -> Self {
+        self.set_body_octet_stream(body);
         self
     }
 
     /// Set the given bytes as the request's body.
     ///
-    /// Any previous body that may have been set on the response is discarded. To add to an existing
-    /// body, use [`get_body_mut()`][`Self::get_body_mut()`] and write to the returned [`Body`].
-    ///
-    /// # Content type
-    ///
-    /// This method sets the content type to `application/octet-stream`.
+    #[doc = include_str!("../../docs/snippets/discards-body.md")]
+    #[doc = include_str!("../../docs/snippets/sets-app-octet-stream.md")]
     ///
     /// # Examples
     ///
     /// ```no_run
     /// # use fastly::Request;
     /// let mut req = Request::post("https://example.com");
-    /// req.set_body_bytes(b"hello, world!");
+    /// req.set_body_octet_stream(b"hello, world!");
     /// assert_eq!(req.get_content_type(), Some(fastly::mime::APPLICATION_OCTET_STREAM));
     /// assert_eq!(&req.into_body_bytes(), b"hello, world!");
     /// ```
-    pub fn set_body_bytes(&mut self, body: &[u8]) {
+    pub fn set_body_octet_stream(&mut self, body: &[u8]) {
         self.body = Some(Body::from(body));
         self.set_content_type(mime::APPLICATION_OCTET_STREAM);
     }
@@ -900,13 +846,7 @@ impl Request {
     ///
     /// After calling this method, this request will no longer have a body.
     ///
-    /// # Memory usage
-    ///
-    /// This method will cause the entire body to be buffering in WebAssembly memory. You should
-    /// take care not to exceed the WebAssembly memory limits, and consider using methods like
-    /// [`read_body_lines()`][`Self::read_body_lines()`] or
-    /// [`read_body_chunks()`][`Self::read_body_chunks()`] to control how much of the body you
-    /// process at once.
+    #[doc = include_str!("../../docs/snippets/buffers-body-reqresp.md")]
     ///
     /// # Examples
     ///
@@ -964,8 +904,7 @@ impl Request {
     /// your own custom type, or use [`serde_json::Value`] to create untyped JSON values. See
     /// [`serde_json`] for details.
     ///
-    /// Any previous body that may have been set on the response is discarded. To add to an existing
-    /// body, use [`get_body_mut()`][`Self::get_body_mut()`] and write to the returned [`Body`].
+    #[doc = include_str!("../../docs/snippets/discards-body.md")]
     ///
     /// # Content type
     ///
@@ -1080,8 +1019,7 @@ impl Request {
     /// The given value must implement [`serde::Serialize`]; see the trait documentation for
     /// details.
     ///
-    /// Any previous body that may have been set on the response is discarded. To add to an existing
-    /// body, use [`get_body_mut()`][`Self::get_body_mut()`] and write to the returned [`Body`].
+    #[doc = include_str!("../../docs/snippets/discards-body.md")]
     ///
     /// # Content type
     ///
@@ -1120,8 +1058,7 @@ impl Request {
     /// Take the request body and attempt to parse it as a `application/x-www-form-urlencoded`
     /// formatted string.
     ///
-    /// The return type chosen for this function must implement [`serde::de::Deserialize`] without
-    /// any non-static lifetimes; see the trait documentation for details.
+    #[doc = include_str!("../../docs/snippets/returns-deserializeowned.md")]
     ///
     /// After calling this method, this request will no longer have a body.
     ///
@@ -1205,10 +1142,7 @@ impl Request {
 
     /// Returns whether the given header name is present in the request.
     ///
-    /// # Argument type conversion
-    ///
-    /// The header name argument can be any type that implements [`ToHeaderName`]; see that trait
-    /// for details on which types can be used and when panics may arise during conversion.
+    #[doc = include_str!("../../docs/snippets/header-name-argument.md")]
     ///
     /// # Examples
     ///
@@ -1234,16 +1168,11 @@ impl Request {
     /// values. See [`get_header_all_str()`][`Self::get_header_all_str()`] if you need to get all of
     /// the values.
     ///
-    /// # Argument type conversion
-    ///
-    /// The header name argument can be any type that implements [`ToHeaderName`]; see that trait
-    /// for details on which types can be used and when panics may arise during conversion.
+    #[doc = include_str!("../../docs/snippets/header-name-argument.md")]
     ///
     /// # Panics
     ///
-    /// This method panics if the value of the header is not a valid UTF-8 string. To handle the
-    /// possibility of invalid UTF-8 data, use [`get_header()`][`Self::get_header()`] and then
-    /// convert the bytes with [`HeaderValue::to_str()`].
+    #[doc = include_str!("../../docs/snippets/panics-reqresp-header-utf8.md")]
     ///
     /// # Examples
     ///
@@ -1270,10 +1199,7 @@ impl Request {
     /// values. See [`get_header_all()`][`Self::get_header_all()`] if you need to get all of the
     /// values.
     ///
-    /// # Argument type conversion
-    ///
-    /// The header name argument can be any type that implements [`ToHeaderName`]; see that trait
-    /// for details on which types can be used and when panics may arise during conversion.
+    #[doc = include_str!("../../docs/snippets/header-name-argument.md")]
     ///
     /// # Examples
     ///
@@ -1300,16 +1226,11 @@ impl Request {
 
     /// Get all values of a header as strings, or an empty vector if the header is not present.
     ///
-    /// # Argument type conversion
-    ///
-    /// The header name argument can be any type that implements [`ToHeaderName`]; see that trait
-    /// for details on which types can be used and when panics may arise during conversion.
+    #[doc = include_str!("../../docs/snippets/header-name-argument.md")]
     ///
     /// # Panics
     ///
-    /// This method panics if any of the header values are not valid UTF-8 strings. To handle the
-    /// possibility of non-UTF-8 data, use [`get_header_all()`][`Self::get_header_all()`] and then
-    /// convert the bytes with [`HeaderValue::to_str()`].
+    #[doc = include_str!("../../docs/snippets/panics-reqresp-headers-utf8.md")]
     ///
     /// # Examples
     ///
@@ -1335,10 +1256,7 @@ impl Request {
 
     /// Get an iterator of all the values of a header.
     ///
-    /// # Argument type conversion
-    ///
-    /// The header name argument can be any type that implements [`ToHeaderName`]; see that trait
-    /// for details on which types can be used and when panics may arise during conversion.
+    #[doc = include_str!("../../docs/snippets/header-name-argument.md")]
     ///
     /// # Examples
     ///
@@ -1436,11 +1354,7 @@ impl Request {
     /// Set a request header to the given value, discarding any previous values for the given
     /// header name.
     ///
-    /// # Argument type conversion
-    ///
-    /// The header name and value arguments can be any types that implement [`ToHeaderName`] and
-    /// [`ToHeaderValue`], respectively. See those traits for details on which types can be used and
-    /// when panics may arise during conversion.
+    #[doc = include_str!("../../docs/snippets/header-name-value-argument.md")]
     ///
     /// # Examples
     ///
@@ -1467,11 +1381,7 @@ impl Request {
     /// Unlike [`set_header()`][`Self::set_header()`], this does not discard existing values for the
     /// same header name.
     ///
-    /// # Argument type conversion
-    ///
-    /// The header name and value arguments can be any types that implement [`ToHeaderName`] and
-    /// [`ToHeaderValue`], respectively. See those traits for details on which types can be used and
-    /// when panics may arise during conversion.
+    #[doc = include_str!("../../docs/snippets/header-name-value-argument.md")]
     ///
     /// # Examples
     ///
@@ -1496,13 +1406,9 @@ impl Request {
     /// Remove all request headers of the given name, and return one of the removed header values
     /// if any were present.
     ///
-    /// If the header has multiple values, one is returned arbitrarily. To get all of the removed
-    /// header values, use [`get_header_all()`][`Self::get_header_all()`] before removing.
+    #[doc = include_str!("../../docs/snippets/removes-one-header.md")]
     ///
-    /// # Argument type conversion
-    ///
-    /// The header name argument can be any type that implements [`ToHeaderName`]; see that trait
-    /// for details on which types can be used and when panics may arise during conversion.
+    #[doc = include_str!("../../docs/snippets/header-name-argument.md")]
     ///
     /// # Examples
     ///
@@ -1521,19 +1427,13 @@ impl Request {
     /// Remove all request headers of the given name, and return one of the removed header values as
     /// a string if any were present.
     ///
-    /// If the header has multiple values, one is returned arbitrarily. To get all of the removed
-    /// header values, use [`get_header_all()`][`Self::get_header_all()`] before removing.
+    #[doc = include_str!("../../docs/snippets/removes-one-header.md")]
     ///
-    /// # Argument type conversion
-    ///
-    /// The header name argument can be any type that implements [`ToHeaderName`]; see that trait
-    /// for details on which types can be used and when panics may arise during conversion.
+    #[doc = include_str!("../../docs/snippets/header-name-argument.md")]
     ///
     /// # Panics
     ///
-    /// This method panics if the value of the header is not a valid UTF-8 string. To handle the
-    /// possibility of invalid UTF-8 data, use [`remove_header()`][`Self::remove_header()`] and then
-    /// convert the bytes with [`HeaderValue::to_str()`].
+    #[doc = include_str!("../../docs/snippets/panics-reqresp-remove-header-utf8.md")]
     ///
     /// # Examples
     ///
@@ -1595,10 +1495,7 @@ impl Request {
 
     /// Set the request method.
     ///
-    /// # Argument type conversion
-    ///
-    /// The method argument can be any type that implements [`ToMethod`]; see that trait for details
-    /// on which types can be used and when panics may arise during conversion.
+    #[doc = include_str!("../../docs/snippets/method-argument.md")]
     ///
     /// # Examples
     ///
@@ -1669,10 +1566,7 @@ impl Request {
 
     /// Set the request URL.
     ///
-    /// # Argument type conversion
-    ///
-    /// The URL argument can be any type that implements [`ToUrl`]; see that trait for details on
-    /// which types can be used and when panics may arise during conversion.
+    #[doc = include_str!("../../docs/snippets/url-argument.md")]
     pub fn set_url(&mut self, url: impl ToUrl) {
         self.url = url.into_owned();
     }
@@ -1718,10 +1612,32 @@ impl Request {
         self.get_url().query()
     }
 
+    /// Get the value of a query parameter in the request's URL.
+    ///
+    /// This assumes that the query string is a `&` separated list of `parameter=value` pairs.  The
+    /// value of the first occurrence of `parameter` is returned. No URL decoding is performed.
+    ///
+    /// # Examples
+    ///
+    /// ```no_run
+    /// # use fastly::Request;
+    /// let req = Request::get("https://example.com/page?foo=bar&baz=qux");
+    /// assert_eq!(req.get_query_parameter("foo"), Some("bar"));
+    /// assert_eq!(req.get_query_parameter("baz"), Some("qux"));
+    /// assert_eq!(req.get_query_parameter("other"), None);
+    /// ```
+    pub fn get_query_parameter(&self, parameter: &str) -> Option<&str> {
+        self.get_url().query().and_then(|qs| {
+            qs.split('&').find_map(|part| {
+                part.strip_prefix(parameter)
+                    .and_then(|maybe| maybe.strip_prefix('='))
+            })
+        })
+    }
+
     /// Attempt to parse the query component of the request URL into the specified datatype.
     ///
-    /// The return type chosen for this function must implement [`serde::de::Deserialize`] without
-    /// any non-static lifetimes; see the trait documentation for details.
+    #[doc = include_str!("../../docs/snippets/returns-deserializeowned.md")]
     ///
     /// # Errors
     ///
@@ -2048,6 +1964,22 @@ impl Request {
         self::handle::client_tls_protocol()
     }
 
+    /// Set whether a `gzip`-encoded response to this request will be automatically decompressed.
+    ///
+    /// If the response to this request is `gzip`-encoded, it will be presented in decompressed
+    /// form, and the `Content-Encoding` and `Content-Length` headers will be removed.
+    pub fn set_auto_decompress_gzip(&mut self, gzip: bool) {
+        self.auto_decompress_response
+            .set(ContentEncodings::GZIP, gzip);
+    }
+
+    /// Builder-style equivalent of
+    /// [`set_auto_decompress_gzip()`][`Self::set_auto_decompress_gzip()`].
+    pub fn with_auto_decompress_gzip(mut self, gzip: bool) -> Self {
+        self.set_auto_decompress_gzip(gzip);
+        self
+    }
+
     /// Create a [`Request`] from the low-level [`handle` API][`crate::handle`].
     ///
     /// # Errors
@@ -2098,11 +2030,13 @@ impl Request {
     fn to_handles(&mut self) -> (RequestHandle, Option<BodyHandle>) {
         let req_handle = {
             let mut req_handle = RequestHandle::new();
-            // Set the handle's version, method, URI, and cache override using the request.
+            // Set the handle's version, method, URI, cache override, and auto decompression
+            // settings using the request.
             req_handle.set_version(self.version);
             req_handle.set_method(&self.method);
             req_handle.set_url(&self.url);
             req_handle.set_cache_override(&self.cache_override);
+            req_handle.set_auto_decompress_response(self.auto_decompress_response);
             for name in self.headers.keys() {
                 // Copy the request's header values to the handle.
                 req_handle.set_header_values(name, self.headers.get_all(name));
@@ -2126,6 +2060,14 @@ impl Request {
         };
         (req_handle, body_handle)
     }
+
+    /// Returns whether or not the client request came with a valid Fastly-Key for the service.
+    pub fn fastly_key_is_valid(&self) -> bool {
+        if !self.is_from_client() {
+            return false;
+        }
+        self::handle::fastly_key_is_valid()
+    }
 }
 
 /// Anything that we need to make a full roundtrip through the `http` types that doesn't have a more
@@ -2134,6 +2076,7 @@ impl Request {
 struct FastlyExts {
     cache_override: CacheOverride,
     is_from_client: bool,
+    auto_decompress_response: ContentEncodings,
     cache_key: Option<CacheKeyGen>,
 }
 
@@ -2143,13 +2086,12 @@ impl Into<http::Request<Body>> for Request {
         req.extensions_mut().insert(FastlyExts {
             cache_override: self.cache_override,
             is_from_client: self.is_from_client,
+            auto_decompress_response: self.auto_decompress_response,
             cache_key: self.cache_key,
         });
         *req.headers_mut() = self.headers;
         *req.method_mut() = self.method;
-        *req.uri_mut() = self
-            .url
-            .into_string()
+        *req.uri_mut() = String::from(self.url)
             .parse()
             .expect("Url to Uri conversion shouldn't fail, but did");
         *req.version_mut() = self.version;
@@ -2163,6 +2105,7 @@ impl From<http::Request<Body>> for Request {
         let FastlyExts {
             cache_override,
             is_from_client,
+            auto_decompress_response,
             cache_key,
         } = parts.extensions.remove().unwrap_or_default();
         Request {
@@ -2174,6 +2117,7 @@ impl From<http::Request<Body>> for Request {
             body: Some(body),
             cache_override,
             is_from_client,
+            auto_decompress_response,
             cache_key,
         }
     }
@@ -2292,8 +2236,7 @@ impl SendError {
 
     /// Get the underlying cause of this `SendError`.
     ///
-    /// This is the same cause that would be returned by
-    /// `err.source().downcast_ref::<SendErrorCause>()`, but more direct.
+    /// This is the same cause that would be returned by `err.source().downcast_ref::<SendErrorCause>()`, but more direct.
     pub fn root_cause(&self) -> &SendErrorCause {
         &self.error
     }
