@@ -1,8 +1,9 @@
 pub use fastly_shared::CacheOverride;
+pub use fastly_sys::ContentEncodings;
 
 use super::pending::handle::PendingRequestHandle;
 use crate::abi::{self, FastlyStatus, MultiValueHostcallError};
-use crate::error::BufferSizeError;
+use crate::error::{BufferSizeError, HandleError, HandleKind};
 use crate::handle::{BodyHandle, ResponseHandle, StreamingBodyHandle};
 use crate::http::request::SendErrorCause;
 use bytes::{BufMut, BytesMut};
@@ -10,6 +11,7 @@ use http::header::{HeaderName, HeaderValue};
 use http::{Method, Uri, Version};
 use lazy_static::lazy_static;
 use std::convert::{TryFrom, TryInto};
+use std::mem::ManuallyDrop;
 use std::net::{IpAddr, Ipv4Addr, Ipv6Addr};
 use std::sync::atomic::{AtomicBool, Ordering};
 use url::Url;
@@ -58,13 +60,13 @@ impl RequestHandle {
     };
 
     /// Returns `true` if the request handle is valid.
-    pub fn is_valid(&self) -> bool {
+    pub const fn is_valid(&self) -> bool {
         !self.is_invalid()
     }
 
     /// Returns `true` if the request handle is invalid.
-    pub fn is_invalid(&self) -> bool {
-        self.handle == Self::INVALID.handle
+    pub const fn is_invalid(&self) -> bool {
+        self.handle == fastly_shared::INVALID_REQUEST_HANDLE
     }
 
     /// Get the underlying representation of the handle.
@@ -83,6 +85,14 @@ impl RequestHandle {
         &mut self.handle
     }
 
+    /// Turn a handle into its representation without closing the underlying resource.
+    ///
+    /// This should only be used when calling the raw ABI directly, and care should be taken not to
+    /// reuse or alias handle values.
+    pub(crate) fn into_u32(self) -> u32 {
+        ManuallyDrop::new(self).as_u32()
+    }
+
     /// Set `GOT_CLIENT_REQ` flag to show we've taken the client request.
     ///
     /// This will panic if the flag has already been set by someone else.
@@ -311,13 +321,7 @@ impl RequestHandle {
         .expect("fastly_http_req::header_values_set failed");
     }
 
-    /// Get the value of a header, or `None` if the header is not present.
-    ///
-    /// If there are multiple values for the header, only one is returned. See
-    /// [`get_header_values()`][`Self::get_header_values()`] if you need to get all of the values.
-    ///
-    /// If the value is longer than `max_len`, this will return a [`BufferSizeError`]; you can retry
-    /// with a larger buffer size if necessary.
+    #[doc = include_str!("../../../docs/snippets/handle-get-header-value.md")]
     pub fn get_header_value(
         &self,
         name: &HeaderName,
@@ -613,8 +617,8 @@ impl RequestHandle {
         let mut resp_body_handle = BodyHandle::INVALID;
         let status = unsafe {
             abi::fastly_http_req::send(
-                self.as_u32(),
-                body.as_u32(),
+                self.into_u32(),
+                body.into_u32(),
                 backend.as_ptr(),
                 backend.len(),
                 resp_handle.as_u32_mut(),
@@ -645,8 +649,8 @@ impl RequestHandle {
         let mut pending_req_handle = PendingRequestHandle::INVALID;
         let status = unsafe {
             abi::fastly_http_req::send_async(
-                self.as_u32(),
-                body.as_u32(),
+                self.into_u32(),
+                body.into_u32(),
                 backend.as_ptr(),
                 backend.len(),
                 pending_req_handle.as_u32_mut(),
@@ -674,7 +678,7 @@ impl RequestHandle {
         let mut pending_req_handle = PendingRequestHandle::INVALID;
         let status = unsafe {
             abi::fastly_http_req::send_async_streaming(
-                self.as_u32(),
+                self.into_u32(),
                 body.as_u32(),
                 backend.as_ptr(),
                 backend.len(),
@@ -687,7 +691,7 @@ impl RequestHandle {
             panic!("fastly_http_req::send_async_streaming returned an invalid handle");
         } else {
             Ok((
-                StreamingBodyHandle::from_body_handle(&body),
+                StreamingBodyHandle::from_body_handle(body),
                 pending_req_handle,
             ))
         }
@@ -716,6 +720,54 @@ impl RequestHandle {
         .result()
         .expect("fastly_http_req::cache_override_v2_set failed");
     }
+
+    /// Close the RequestHandle by removing it from the host Session. If the handle has already
+    /// been closed an error will be returned. When calling
+    /// send/send_async/send_async_streaming the RequestHandle is consumed and
+    /// it's cleaned up. You should only call `close` if you have not sent a
+    /// request yet and want to clean up the resources if not being used.
+    ///
+    /// # Examples
+    ///
+    /// ```no_run
+    /// # use fastly::handle::RequestHandle;
+    /// # fn main() -> Result<(), Box<dyn std::error::Error>> {
+    /// let request = RequestHandle::new();
+    /// // The handle is not being used so we can close it out without any
+    /// // trouble
+    /// request.close()?;
+    /// # Ok(())
+    /// # }
+    /// ```
+    pub fn close(self) -> Result<(), HandleError> {
+        match unsafe { abi::fastly_http_req::close(self.as_u32()) } {
+            FastlyStatus::OK => Ok(()),
+            _ => Err(HandleError::ClosedHandle(HandleKind::Request)),
+        }
+    }
+
+    /// Set the content encodings to automatically decompress responses to this request.
+    ///
+    /// If the response to this request is encoded by one of the encodings set by this method, the
+    /// response will be presented to the Compute@Edge program in decompressed form with the
+    /// `Content-Encoding` and `Content-Length` headers removed.
+    pub fn set_auto_decompress_response(&mut self, content_encodings: ContentEncodings) {
+        unsafe {
+            abi::fastly_http_req::auto_decompress_response_set(self.as_u32(), content_encodings)
+        }
+        .result()
+        .expect("fastly_http_req::auto_decompress_response_set failed")
+    }
+}
+
+impl Drop for RequestHandle {
+    fn drop(&mut self) {
+        if self.is_valid() {
+            unsafe { abi::fastly_http_req::close(self.as_u32()) }
+                .result()
+                .expect("fastly_http_req::close failed");
+        }
+    }
 }
 
 /// Get handles to the client request headers and body at the same time.
@@ -803,6 +855,18 @@ pub fn client_original_header_count() -> u32 {
     count
 }
 
+/// Returns whether or not a valid Fastly-Key for this service was received.
+pub fn fastly_key_is_valid() -> bool {
+    let mut is_valid = 0;
+    let status = unsafe { abi::fastly_http_req::fastly_key_is_valid(&mut is_valid) };
+    if status.is_err() {
+        panic!("fastly_key_is_valid failed")
+    }
+    // Just in case more information needs to be conveyed by fastly_key_is_valid in the future, we
+    // can at least establish that any authenticated key will have at least the lowest bit set.
+    (is_valid & 1) != 0
+}
+
 /// Returns the IP address of the client making the HTTP request.
 pub fn client_ip_addr() -> Option<IpAddr> {
     let mut octets = [0; 16];
