@@ -38,8 +38,7 @@ impl PendingRequest {
     ///
     /// # Panics
     ///
-    /// Panics if the backend response exceeds any of the limits configured by
-    /// [`ResponseLimits`][`crate::limits::ResponseLimits`].
+    #[doc = include_str!("../../../docs/snippets/panics-responselimits.md")]
     pub fn poll(self) -> PollResult {
         let Self { handle, metadata } = self;
         match handle.copy().poll() {
@@ -61,8 +60,7 @@ impl PendingRequest {
     ///
     /// # Panics
     ///
-    /// Panics if the backend response exceeds any of the limits configured by
-    /// [`ResponseLimits`][`crate::limits::ResponseLimits`].
+    #[doc = include_str!("../../../docs/snippets/panics-responselimits.md")]
     pub fn wait(self) -> Result<Response, SendError> {
         let (resp_handle, resp_body_handle) =
             try_with_pending_req!(self, self.handle.copy().wait());
@@ -181,8 +179,7 @@ pub enum PollResult {
 /// Panics if the argument collection is empty, or contains more than
 /// [`fastly_shared::MAX_PENDING_REQS`] requests.
 ///
-/// Panics if the backend response exceeds any of the limits configured by
-/// [`ResponseLimits`][`crate::limits::ResponseLimits`].
+#[doc = include_str!("../../../docs/snippets/panics-responselimits.md")]
 pub fn select<I>(pending_reqs: I) -> (Result<Response, SendError>, Vec<PendingRequest>)
 where
     I: IntoIterator<Item = PendingRequest>,
