@@ -146,9 +146,7 @@ impl Response {
     /// This method requires mutable access to this response because reading from and writing to the
     /// body can involve an HTTP connection.
     ///
-    /// This operation is potentially expensive if the body is large. Take care when using this
-    /// method on bodies with unknown sizes. Consider using methods like [`BufRead::lines()`] or
-    /// [`Body::read_chunks()`] to incrementally process a body while limiting the maximum size.
+    #[doc = include_str!("../../docs/snippets/clones-body.md")]
     ///
     /// # Examples
     ///
@@ -177,9 +175,7 @@ impl Response {
 
     /// Create a new [`Response`] with the given value as the body.
     ///
-    /// # Argument type conversion
-    ///
-    /// See [`Body`] for details on which types can be used as a body.
+    #[doc = include_str!("../../docs/snippets/body-argument.md")]
     ///
     /// # Examples
     ///
@@ -201,9 +197,7 @@ impl Response {
 
     /// Create a new response with the given status code.
     ///
-    /// # Argument type conversion
-    ///
-    /// See [`Body`] for details on which types can be used as a body.
+    #[doc = include_str!("../../docs/snippets/body-argument.md")]
     ///
     /// # Examples
     ///
@@ -288,8 +282,7 @@ impl Response {
 
     /// Get a mutable reference to the body of this response.
     ///
-    /// An empty body is returned if no body has been set, or if it has previously been returned by
-    /// a method like [`take_body()`][`Self::take_body()`].
+    #[doc = include_str!("../../docs/snippets/creates-empty-body.md")]
     ///
     /// # Examples
     ///
@@ -359,13 +352,8 @@ impl Response {
     }
 
     /// Set the given value as the response's body.
-    ///
-    /// Any previous body that may have been set on the response is discarded. To add to an existing
-    /// body, use [`get_body_mut()`][`Self::get_body_mut()`] and write to the returned [`Body`].
-    ///
-    /// # Argument type conversion
-    ///
-    /// See [`Body`] for details on which types can be used as a body.
+    #[doc = include_str!("../../docs/snippets/body-argument.md")]
+    #[doc = include_str!("../../docs/snippets/discards-body.md")]
     pub fn set_body(&mut self, body: impl Into<Body>) {
         self.body = Some(body.into());
     }
@@ -374,8 +362,7 @@ impl Response {
     ///
     /// After calling this method, this response will no longer have a body.
     ///
-    /// An empty body is returned if no body has been set, or if it has previously been returned by
-    /// a method like [`take_body()`][`Self::take_body()`].
+    #[doc = include_str!("../../docs/snippets/creates-empty-body.md")]
     pub fn take_body(&mut self) -> Body {
         self.body.take().unwrap_or_else(|| Body::new())
     }
@@ -392,8 +379,7 @@ impl Response {
     ///
     /// If this response does not have a body, the appended body is set as the response's body.
     ///
-    /// This operation is performed in amortized constant time, and so should always be preferred to
-    /// reading an entire body and then writing the same contents to another body.
+    #[doc = include_str!("../../docs/snippets/body-append-constant-time.md")]
     ///
     /// This method should be used when combining bodies that have not necessarily been read yet,
     /// such as a body returned from a backend response. To append contents that are already in
@@ -419,13 +405,8 @@ impl Response {
 
     /// Consume the response and return its body as a byte vector.
     ///
-    /// # Memory usage
-    ///
-    /// This method will cause the entire body to be buffering in WebAssembly memory. You should
-    /// take care not to exceed the WebAssembly memory limits, and consider using methods like
-    /// [`read_body_lines()`][`Self::read_body_lines()`] or
-    /// [`read_body_chunks()`][`Self::read_body_chunks()`] to control how much of the body you
-    /// process at once.  # Examples
+    #[doc = include_str!("../../docs/snippets/buffers-body-reqresp.md")]
+    /// # Examples
     ///
     /// ```no_run
     /// # use fastly::Response;
@@ -438,20 +419,11 @@ impl Response {
 
     /// Consume the response and return its body as a string.
     ///
-    /// # Memory usage
-    ///
-    /// This method will cause the entire body to be buffering in WebAssembly memory. You should
-    /// take care not to exceed the WebAssembly memory limits, and consider using methods like
-    /// [`read_body_lines()`][`Self::read_body_lines()`] or
-    /// [`read_body_chunks()`][`Self::read_body_chunks()`] to control how much of the body you
-    /// process at once.
+    #[doc = include_str!("../../docs/snippets/buffers-body-reqresp.md")]
     ///
     /// # Panics
     ///
-    /// If the body does not contain a valid UTF-8 string, this function will panic. To explicitly
-    /// handle the possibility of invalid UTF-8 data, use
-    /// [`into_body_bytes()`][`Self::into_body_bytes()`] and then convert the bytes explicitly with
-    /// a function like [`String::from_utf8`].
+    #[doc = include_str!("../../docs/snippets/panics-reqresp-intobody-utf8.md")]
     ///
     /// # Examples
     ///
@@ -467,8 +439,7 @@ impl Response {
 
     /// Consume the response and return its body.
     ///
-    /// An empty body is returned if no body has been set, or if it has previously been returned by
-    /// a method like [`take_body()`][`Self::take_body()`].
+    #[doc = include_str!("../../docs/snippets/creates-empty-body.md")]
     pub fn into_body(self) -> Body {
         self.body.unwrap_or_else(|| Body::new())
     }
@@ -501,12 +472,8 @@ impl Response {
 
     /// Set the given string as the response's body with content type `text/plain; charset=UTF-8`.
     ///
-    /// Any previous body that may have been set on the response is discarded. To add to an existing
-    /// body, use [`get_body_mut()`][`Self::get_body_mut()`] and write to the returned [`Body`].
-    ///
-    /// # Content type
-    ///
-    /// This method sets the content type to `text/plain; charset=utf-8`.
+    #[doc = include_str!("../../docs/snippets/discards-body.md")]
+    #[doc = include_str!("../../docs/snippets/sets-text-plain.md")]
     ///
     /// # Examples
     ///
@@ -530,12 +497,8 @@ impl Response {
 
     /// Set the given string as the request's body with content type `text/html; charset=UTF-8`.
     ///
-    /// Any previous body that may have been set on the response is discarded. To add to an existing
-    /// body, use [`get_body_mut()`][`Self::get_body_mut()`] and write to the returned [`Body`].
-    ///
-    /// # Content type
-    ///
-    /// This method sets the content type to `text/html; charset=utf-8`.
+    #[doc = include_str!("../../docs/snippets/discards-body.md")]
+    #[doc = include_str!("../../docs/snippets/sets-text-html.md")]
     ///
     /// # Examples
     ///
@@ -555,19 +518,11 @@ impl Response {
     ///
     /// After calling this method, this response will no longer have a body.
     ///
-    /// # Memory usage
-    ///
-    /// This method will cause the entire body to be buffering in WebAssembly memory. You should
-    /// take care not to exceed the WebAssembly memory limits, and consider using methods like
-    /// [`read_body_lines()`][`Self::read_body_lines()`] or
-    /// [`read_body_chunks()`][`Self::read_body_chunks()`] to control how much of the body you
-    /// process at once.
+    #[doc = include_str!("../../docs/snippets/buffers-body-reqresp.md")]
     ///
     /// # Panics
     ///
-    /// If the body does not contain a valid UTF-8 string, this function will panic. To handle the
-    /// possibility of invalid UTF-8 data, use [`take_body_bytes()`][`Self::take_body_bytes()`] and
-    /// then convert the bytes with a function like [`String::from_utf8`].
+    #[doc = include_str!("../../docs/snippets/panics-reqresp-takebody-utf8.md")]
     ///
     /// # Examples
     ///
@@ -606,31 +561,45 @@ impl Response {
         self.get_body_mut().lines()
     }
 
-    /// Builder-style equivalent of [`set_body_bytes()`][`Self::set_body_bytes()`].
-    pub fn with_body_bytes(mut self, body: &[u8]) -> Self {
-        self.set_body_bytes(body);
+    /// Deprecated alias of [`with_body_octet_stream()`][`Self::with_body_octet_stream()`]
+    #[deprecated(
+        since = "0.8.1",
+        note = "renamed to `Response::with_body_octet_stream()`"
+    )]
+    pub fn with_body_bytes(self, body: &[u8]) -> Self {
+        self.with_body_octet_stream(body)
+    }
+
+    /// Deprecated alias of [`set_body_octet_stream()`][`Self::set_body_octet_stream()`]
+    #[deprecated(
+        since = "0.8.1",
+        note = "renamed to `Response::set_body_octet_stream()`"
+    )]
+    pub fn set_body_bytes(&mut self, body: &[u8]) {
+        self.set_body_octet_stream(body);
+    }
+
+    /// Builder-style equivalent of [`set_body_octet_stream()`][`Self::set_body_octet_stream()`].
+    pub fn with_body_octet_stream(mut self, body: &[u8]) -> Self {
+        self.set_body_octet_stream(body);
         self
     }
 
     /// Set the given bytes as the response's body.
     ///
-    /// Any previous body that may have been set on the response is discarded. To add to an existing
-    /// body, use [`get_body_mut()`][`Self::get_body_mut()`] and write to the returned [`Body`].
-    ///
-    /// # Content type
-    ///
-    /// This method sets the content type to `application/octet-stream`.
+    #[doc = include_str!("../../docs/snippets/discards-body.md")]
+    #[doc = include_str!("../../docs/snippets/sets-app-octet-stream.md")]
     ///
     /// # Examples
     ///
     /// ```no_run
     /// # use fastly::Response;
     /// let mut resp = Response::new();
-    /// resp.set_body_bytes(b"hello, world!");
+    /// resp.set_body_octet_stream(b"hello, world!");
     /// assert_eq!(resp.get_content_type(), Some(fastly::mime::APPLICATION_OCTET_STREAM));
     /// assert_eq!(&resp.into_body_bytes(), b"hello, world!");
     /// ```
-    pub fn set_body_bytes(&mut self, body: &[u8]) {
+    pub fn set_body_octet_stream(&mut self, body: &[u8]) {
         self.body = Some(Body::from(body));
         self.set_content_type(mime::APPLICATION_OCTET_STREAM);
     }
@@ -639,13 +608,7 @@ impl Response {
     ///
     /// After calling this method, this response will no longer have a body.
     ///
-    /// # Memory usage
-    ///
-    /// This method will cause the entire body to be buffering in WebAssembly memory. You should
-    /// take care not to exceed the WebAssembly memory limits, and consider using methods like
-    /// [`read_body_lines()`][`Self::read_body_lines()`] or
-    /// [`read_body_chunks()`][`Self::read_body_chunks()`] to control how much of the body you
-    /// process at once.
+    #[doc = include_str!("../../docs/snippets/buffers-body-reqresp.md")]
     ///
     /// # Examples
     ///
@@ -703,12 +666,8 @@ impl Response {
     /// your own custom type, or use [`serde_json::Value`] to create untyped JSON values. See
     /// [`serde_json`] for details.
     ///
-    /// Any previous body that may have been set on the response is discarded. To add to an existing
-    /// body, use [`get_body_mut()`][`Self::get_body_mut()`] and write to the returned [`Body`].
-    ///
-    /// # Content type
-    ///
-    /// This method sets the content type to `application/json`.
+    #[doc = include_str!("../../docs/snippets/discards-body.md")]
+    #[doc = include_str!("../../docs/snippets/sets-app-json.md")]
     ///
     /// # Errors
     ///
@@ -817,8 +776,7 @@ impl Response {
     /// The given value must implement [`serde::Serialize`]; see the trait documentation for
     /// details.
     ///
-    /// Any previous body that may have been set on the response is discarded. To add to an existing
-    /// body, use [`get_body_mut()`][`Self::get_body_mut()`] and write to the returned [`Body`].
+    #[doc = include_str!("../../docs/snippets/discards-body.md")]
     ///
     /// # Content type
     ///
@@ -857,8 +815,7 @@ impl Response {
     /// Take the response body and attempt to parse it as a `application/x-www-form-urlencoded`
     /// formatted string.
     ///
-    /// The return type chosen for this function must implement [`serde::de::Deserialize`] without
-    /// any non-static lifetimes; see the trait documentation for details.
+    #[doc = include_str!("../../docs/snippets/returns-deserializeowned.md")]
     ///
     /// After calling this method, this response will no longer have a body.
     ///
@@ -940,9 +897,7 @@ impl Response {
 
     /// Returns whether the given header name is present in the response.
     ///
-    /// # Argument type conversion
-    ///
-    /// The header name argument can be any type that implements [`ToHeaderName`]; see that trait for details on which types can be used and when panics may arise during conversion.
+    #[doc = include_str!("../../docs/snippets/header-name-argument.md")]
     ///
     /// # Examples
     ///
@@ -968,16 +923,11 @@ impl Response {
     /// values. See [`get_header_all_str()`][`Self::get_header_all_str()`] if you need to get all of
     /// the values.
     ///
-    /// # Argument type conversion
-    ///
-    /// The header name argument can be any type that implements [`ToHeaderName`]; see that trait
-    /// for details on which types can be used and when panics may arise during conversion.
+    #[doc = include_str!("../../docs/snippets/header-name-argument.md")]
     ///
     /// # Panics
     ///
-    /// This method panics if the value of the header is not a valid UTF-8 string. To handle the
-    /// possibility of invalid UTF-8 data, use [`get_header()`][`Self::get_header()`] and then
-    /// convert the bytes with [`HeaderValue::to_str()`].
+    #[doc = include_str!("../../docs/snippets/panics-reqresp-header-utf8.md")]
     ///
     /// # Examples
     ///
@@ -1005,10 +955,7 @@ impl Response {
     /// values. See [`get_header_all()`][`Self::get_header_all()`] if you need to get all of the
     /// values.
     ///
-    /// # Argument type conversion
-    ///
-    /// The header name argument can be any type that implements [`ToHeaderName`]; see that trait
-    /// for details on which types can be used and when panics may arise during conversion.
+    #[doc = include_str!("../../docs/snippets/header-name-argument.md")]
     ///
     /// # Examples
     ///
@@ -1035,16 +982,11 @@ impl Response {
 
     /// Get all values of a header as strings, or an empty vector if the header is not present.
     ///
-    /// # Argument type conversion
-    ///
-    /// The header name argument can be any type that implements [`ToHeaderName`]; see that trait
-    /// for details on which types can be used and when panics may arise during conversion.
+    #[doc = include_str!("../../docs/snippets/header-name-argument.md")]
     ///
     /// # Panics
     ///
-    /// This method panics if any of the header values are not valid UTF-8 strings. To handle the
-    /// possibility of non-UTF-8 data, use [`get_header_all()`][`Self::get_header_all()`] and then
-    /// convert the bytes with [`HeaderValue::to_str()`].
+    #[doc = include_str!("../../docs/snippets/panics-reqresp-headers-utf8.md")]
     ///
     /// # Examples
     ///
@@ -1070,10 +1012,7 @@ impl Response {
 
     /// Get an iterator of all the values of a header.
     ///
-    /// # Argument type conversion
-    ///
-    /// The header name argument can be any type that implements [`ToHeaderName`]; see that trait
-    /// for details on which types can be used and when panics may arise during conversion.
+    #[doc = include_str!("../../docs/snippets/header-name-argument.md")]
     ///
     /// # Examples
     ///
@@ -1174,11 +1113,7 @@ impl Response {
     /// Set a response header to the given value, discarding any previous values for the given
     /// header name.
     ///
-    /// # Argument type conversion
-    ///
-    /// The header name and value arguments can be any types that implement [`ToHeaderName`] and
-    /// [`ToHeaderValue`], respectively. See those traits for details on which types can be used and
-    /// when panics may arise during conversion.
+    #[doc = include_str!("../../docs/snippets/header-name-value-argument.md")]
     ///
     /// # Examples
     ///
@@ -1205,11 +1140,7 @@ impl Response {
     /// Unlike [`set_header()`][`Self::set_header()`], this does not discard existing values for the
     /// same header name.
     ///
-    /// # Argument type conversion
-    ///
-    /// The header name and value arguments can be any types that implement [`ToHeaderName`] and
-    /// [`ToHeaderValue`], respectively. See those traits for details on which types can be used and
-    /// when panics may arise during conversion.
+    #[doc = include_str!("../../docs/snippets/header-name-value-argument.md")]
     ///
     /// # Examples
     ///
@@ -1235,13 +1166,11 @@ impl Response {
     /// Remove all response headers of the given name, and return one of the removed header values
     /// if any were present.
     ///
-    /// If the header has multiple values, one is returned arbitrarily. To get all of the removed header
-    /// values, use [`get_header_all()`][`Self::get_header_all()`] before removing.
+    /// If the header has multiple values, one is returned arbitrarily. To get all of the removed
+    /// header values, or to get a specific value, use
+    /// [`get_header_all()`][`Self::get_header_all()`].
     ///
-    /// # Argument type conversion
-    ///
-    /// The header name argument can be any type that implements [`ToHeaderName`]; see that trait
-    /// for details on which types can be used and when panics may arise during conversion.
+    #[doc = include_str!("../../docs/snippets/header-name-argument.md")]
     ///
     /// # Examples
     ///
@@ -1260,19 +1189,13 @@ impl Response {
     /// Remove all response headers of the given name, and return one of the removed header values
     /// as a string if any were present.
     ///
-    /// If the header has multiple values, one is returned arbitrarily. To get all of the removed
-    /// header values, use [`get_header_all()`][`Self::get_header_all()`] before removing.
+    #[doc = include_str!("../../docs/snippets/removes-one-header.md")]
     ///
-    /// # Argument type conversion
-    ///
-    /// The header name argument can be any type that implements [`ToHeaderName`]; see that trait
-    /// for details on which types can be used and when panics may arise during conversion.
+    #[doc = include_str!("../../docs/snippets/header-name-argument.md")]
     ///
     /// # Panics
     ///
-    /// This method panics if the value of the header is not a valid UTF-8 string. To handle the
-    /// possibility of invalid UTF-8 data, use [`remove_header()`][`Self::remove_header()`] and then
-    /// convert the bytes with [`HeaderValue::to_str()`].
+    #[doc = include_str!("../../docs/snippets/panics-reqresp-remove-header-utf8.md")]
     ///
     /// # Examples
     ///
@@ -1309,10 +1232,7 @@ impl Response {
 
     /// Set the HTTP status code of the response.
     ///
-    /// # Argument type conversion
-    ///
-    /// The status code argument can be any type that implements [`ToStatusCode`]; see that trait
-    /// for details on which types can be used and when panics may arise during conversion.
+    #[doc = include_str!("../../docs/snippets/statuscode-argument.md")]
     ///
     /// # Examples
     ///
