@@ -3,9 +3,9 @@
 #![doc(test(attr(allow(dead_code))))]
 #![doc(test(attr(allow(unused_variables))))]
 #![warn(missing_docs)]
-#![deny(broken_intra_doc_links)]
+#![deny(rustdoc::broken_intra_doc_links)]
 #![deny(private_intra_doc_links)]
-#![deny(invalid_codeblock_attributes)]
+#![deny(rustdoc::invalid_codeblock_attributes)]
 
 //! # Rust SDK for Compute@Edge.
 //!
@@ -141,7 +141,7 @@ mod abi;
 
 mod backend;
 pub mod convert;
-mod dictionary;
+pub mod dictionary;
 pub mod error;
 pub mod experimental;
 pub mod geo;
@@ -152,6 +152,7 @@ pub mod log;
 pub mod mime;
 
 pub use crate::backend::Backend;
+#[doc(inline)]
 pub use crate::dictionary::Dictionary;
 #[doc(inline)]
 pub use crate::error::Error;
